@@ -10,7 +10,7 @@ use Text::CSV;
 my $csv = Text::CSV->new;
 
 open(my $in, '<', 'ipv4-recovered-address-space-2.csv');
-open(my $out, '<', 'ip_del_recovered.h');
+open(my $out, '>', 'ip_del_recovered.h');
 
 while (my $row = $csv->getline($in)) {
 	next if $row->[0] eq 'Start address';
